#!/bin/sh
#
# Checks if a study is deletable (purgable) from the worklist
#
# Usage:
#    isStudyDeletable.sh [-f] -s <study>
#
#      -f         : do full check. Starting without -f only forward to external archive references will be checked
#      -s <study> : study instance uid of the study to be checked
#
#  In default only external archive references will be checked, ie. if the study has been sent to the configured
#  external archive, or if forward tasks to external archive have been queued.
#
#  Full check will also check if in case a study delete:
#      - all objects in study directory would be deleted
#      - study directory would be deleted
#
#  Full check will print diagnostic to standard output:
#    None       - study directory with objects would be deleted (none to remain)
#    Directory  - study objects would be deleted, but study directory would remain
#    ArchiveFwd - study objects would remain with forward to external archive references only.
#    Objects    - study objects would remain
#
# Exit codes in default mode:
#   0 - deletable
#   1 - not deletable
#   2 - error
#
# Exit codes in full mode:
#   0  - success
#   >0 - error
#
# Currently it only checks if any object of the study has 'commit_dev_${ExternalStore}' reference
# without 'fwd_${ExternalStore}' reference.
#

. $HOME/etc/setenv.sh

Study=""
FullCheck="false"
ERRMSG=""

Archive="no"
[ -f "${ARCHIVE_VAR}/archive.cfg" ] && . ${ARCHIVE_VAR}/archive.cfg

#
# Checks External Archive references
#
checkExternalArchive_() {
	# Only check if External Archive is configured
	[ "$Archive" != "external" ] && return 0
	
	HASHDIR="`~/component/utils/bin/getHashDirectory -i $Study`"
	referencesNew=`grep -l -r "^REFERRER=\"commit_dev_${ExternalStore}\"" ${REFCOUNT_VAR}/OBJECTFILE/${HASHDIR}/ 2>/dev/null`
	referencesOld=`grep -l -r "^REFERRER=\"commit_dev_${ExternalStore}\"" ${REFCOUNT_VAR}/OBJECTFILE/${Study}/ 2>/dev/null`

	# There are no commit_dev_ references -> OK
	[ -z "$referencesOld" -a -z "$referencesNew" ] && return 0

	for f in $referencesOld ; do
		if ! grep -q "^REFERRER=\"forward_${ExternalStore}\"" $f 2>/dev/null; then
			return 1
		fi
	done

	for f in $referencesNew ; do
		if ! grep -q "^REFERRER=\"forward_${ExternalStore}\"" $f 2>/dev/null; then
			return 1
		fi
	done
	
	# There are forward_ references for each commit_dev_ references
	return 0
}

# Check StudyDb references
#
# return :
#       0 - only database ref.
#		1 - other references present.
checkStudyDb_() {
	NOTDATABASEREFS=`~/component/utils/bin/refcount -p -i ${Study} -t STUDYDB 2>/dev/null | grep -v -e "database" -e "link_" 2>/dev/null | wc -l`;
	if [ $NOTDATABASEREFS -gt 0 ]; then
		return 1;
	fi
	return 0;
}

#
# Checks objects references
#
#
# return:
#  0 - has exactly one database reference only
#  1 - has only commit_dev_${ExternalStore} and forward_${ExternalStore} references (and exactly one database reference)
#  2 - has other references
#  3 - error (there is no reference in the reference counter)
#
checkObjectFiles_() {
	HASHDIR="`~/component/utils/bin/getHashDirectory -i $Study`"
	
	HASHEDFILES=`ls -1 ${REFCOUNT_VAR}/OBJECTFILE/$HASHDIR 2>/dev/null`;
	HASHEDFILESWITHPATH=`sed {s:^:$REFCOUNT_VAR/OBJECTFILE/$HASHDIR:g} <<<"$HASHEDFILES"`;
	SIMPLEFILES=`ls -1 ${REFCOUNT_VAR}/OBJECTFILE/${Study}/ 2>/dev/null`;	
	[ -n "$SIMPLEFILES" ] && SIMPLEFILESWITHPATH=`sed {s:^:${REFCOUNT_VAR}/OBJECTFILE/$Study/:g} <<<"$SIMPLEFILES"`;
	FILESWITHPATH=${HASHEDFILESWITHPATH}' '${SIMPLEFILESWITHPATH};

	DATABASEREFS=`echo $FILESWITHPATH | xargs grep -e "REFERRER" -e database 2>/dev/null`;
	DATABASEREFSCOUNT=`wc -l <<<"$DATABASEREFS"`;
	DATABASEREFSUNIQCOUNT=`sort <<<"$DATABASEREFS" | uniq | wc -l `;
	if [ $DATABASEREFSCOUNT -ne $DATABASEREFSUNIQCOUNT ];
	then
		return 3
	fi

	OTHERREFSCOUNT=`echo $FILESWITHPATH | xargs grep -v -e ^TYPE -e database -e commit_dev_${ExternalStore} -e forward_${ExternalStore} 2>/dev/null  | wc -l`;
	
	if [ $OTHERREFSCOUNT -gt 0 ];
	then
		return 2
	fi

	COMMITANDFORWARD=`echo $FILESWITHPATH | xargs grep -E "(commit_dev_${ExternalStore})|(forward_${ExternalStore})" 2>/dev/null | wc -l`;
	if [ $COMMITANDFORWARD -gt 0 ];
	then
		return 1
	fi

	return 0;	
}

#
# Checks StudyDir references
#
checkStudyDir_() {	
	REFS=`~/component/utils/bin/refcount -p -i ${Study} -t STUDYDIR 2>/dev/null | sort`;
	
	HASHDIR="`~/component/utils/bin/getHashDirectory -i $Study`"

	FILES=`ls -1 ${REFCOUNT_VAR}/OBJECTFILE/$HASHDIR ${REFCOUNT_VAR}/OBJECTFILE/$Study/ 2>/dev/null | grep -v -e "^$" -e "^/" | sort`;
	
	if [ "$FILES" == "$REFS" ];
	then
		return 0
	fi
		
	return 1;
}

#
# Performs all checks and prints the result.
#
fullCheck_() {
	checkStudyDb_;
	chkDb=$?;
	checkObjectFiles_;
	chkObjs=$?;
	checkStudyDir_;
	chkDir=$?;

	if [ $chkDb -eq 0 ] && [ $chkObjs -eq 0 ] && [ $chkDir -eq 0 ];
	then
		echo -n None
	elif [ $chkDb -eq 0 ] && [ $chkObjs -eq 0 ] && [ $chkDir -eq 1 ];
	then
		echo -n Directory
	elif [ $chkDb -eq 0 ] && [ $chkObjs -eq 1 ];
	then
		echo -n ArchiveFwd
	elif [ $chkDb -eq 0 ] && [ $chkObjs -eq 2 ];
	then
		echo -n Objects
	else
		return 1;
	fi;
}

while getopts 'fs:' OPT; do
	case $OPT in
	f) FullCheck="true" ;;
	s) Study="$OPTARG" ;;
	*) ERRMSG="Invalid Option: $OPT" ;;
	esac
done

if [ -z "$Study" ] ; then
	echo "Missing Option: '-s <study>'" >&2
	exit 2
elif [ -n "$ERRMSG" ] ; then
	echo "$ERRMSG" >&2
	exit 2
fi

if [ "$FullCheck" != "true" ] ; then
	checkExternalArchive_
	exit $?
else
	fullCheck_;
	exit $?;
fi

exit 2

