#!/bin/sh
#
# Component: cd
#
# Description:
#   Prepares studies and creates iso images for writing studies to CD
#
# Usage:
#    createcd [ -d tmpdir ] [-v [ -p productTypePostfix ] [ -l languageFile ] ] [-s] [-S] study ...
#

. $HOME/etc/setenv.sh


#
# $1 error message to be added to the done file
#
function errorExit_() {
	local errMessage="$1"
	
	echo "ERROR: $errMessage" > $workDir/cdimage.done
	exit 1
}


function usage_() {
	local errorMessage="$1"

	[ -n "$errorMessage" ] && echo "$errorMessage" >&2
	cat <<-EOFUSAGE
		createcd -d tmpdir [-v [ -l languageFile ] ] [-s] [-t mediaType] [-c compressionMode]

		-d workdir:     working directory
		-v :            add viewer to the DICOM CD
		-l languageFile: language resources file to add for the viewer 
		-s :            exclude report fiels from DICOM CD
		-t :            media type: CD/CF/DVD/DVD-RAM/MMC/SD/USB (default: CD)
		-c :            compression mode: JPEG/JP2K/Uncompressed/AsIs (default: AsIs)
		-i mediaid:     file-set ID to be added to the dicomdir
		-m md5:         md5 sum to be added to the README
		-u user:        ID of the user creating the media
		-U profileUser: ID of the user whose profile should be added to the media
		
		Objects to be added to the media intended to be collected to \$workdir/studies prior to calling createcd  
	EOFUSAGE

	[ -n "$errorMessage" ] && exit 1
	exit 0
}


addViewer=""
excludeReports=""
mediaType="CD"
compressionMode="AsIs"
mediaID=""
md5Sum=""
userID=""
profileUser=""
workDir=""
languageFile=""

while getopts 'p:d:vst:c:i:m:u:U:l:' OPT; do
	case $OPT in
	s) excludeReports="true" ;;
	c) compressionMode="$OPTARG" ;;
	t) mediaType="$OPTARG" ;;
	d) workDir="$OPTARG" ;;
	i) mediaID="$OPTARG" ;;
	m) md5Sum="$OPTARG" ;;
	u) userID="$OPTARG" ;;
	U) profileUser="$OPTARG" ;;
	v) addViewer="true" ;;
	l) languageFile="$OPTARG" ;;
	*) usage_ "Invalid Option: $OPT" ;;
	esac
done

[ ! -d "$workDir" ] && errorExit_ "Working directory does not exist"
[ ! -d "$workDir/studies" ] && errorExit_ "Studies directory does not exist"

echo "$$" > "$workDir/pid"

contentDir="$workDir/content"
mkdir "${contentDir}"
[ $? -ne 0 ] && errorExit_ "Content directory could not be created"


cd "$workDir"
[ $? -ne 0 ] && errorExit_ "Could not change current directory to $workDir"



# Reading settings from sessionparams file.
# Currently only Compression and Studies is used, but others may also needed eventually (eg currently passed in options)
# 
Extension=""
ObjectCount=""
MediaType=""
Compression=""
Server=""
ProfileUser=""
Date=""
Time=""
ExcludeReports=""
User=""
Studies=""
AddViewer=""
StudyCount=""

[ -f "$workDir/sessionparams" ] && . "$workDir/sessionparams" 2>/dev/null

# compressionMode (-c option) indicates whether the objects need to be decompressed or were already decompressed at HUB servers
# Compression (sessionparams) is the compression mode requested by the user

sCnt=1;

if [ "$compressionMode" = "Uncompressed" ] ; then
	echo "Decompressing studies"
		
	for study in `ls "$workDir/studies"`; do
		printf -v styname "S%07d" $sCnt
		echo "Decompressing study $study to $styname"
		sCnt=`expr $sCnt + 1`

		mkdir "$contentDir/$styname"
		if [ $? -ne 0 ] ; then
			errorExit_ "Error creating $styname directory for study $study"
		fi
		
		for fname in `ls "$workDir/studies/$study"` ; do
			if [ "$fname" == "_REPORT.PDF" ] ; then
				cp "$workDir/studies/$study/$fname" "$contentDir/$styname/$fname"
				continue
			fi
			echo "Decompressing file: $fname"
			echo "$UTILS_HOME/bin/convert +te $workDir/studies/$study/$fname $contentDir/$styname/$fname"
			eval $UTILS_HOME/bin/convert  +te "$workDir/studies/$study/$fname" "$contentDir/$styname/$fname" 2>&1
			if [ $? != 0 ] ; then
				errorExit_ "Error converting $study/$fname to Little Endian Explicit TS"
				#cp "$workDir/studies/$study/$fname" "$contentDir/$styname/$fname"
				#if [ $? -ne 0 ] ; then
				#	errorExit_ "Error copying $study/$fname"
				#fi
			fi
			
			rm "$workDir/studies/$study/$fname"
		done

		rmdir "$workDir/studies/$study"
	done

else
	echo "Moving prepared studies to content directory"
	
	for study in `ls "$workDir/studies"` ; do
		printf -v styname "S%07d" $sCnt
		echo "Moving study $study to $styname"
		sCnt=`expr $sCnt + 1`
		
		mv "$workDir/studies/$study" "$contentDir/$styname"
		[ $? -ne 0 ] && errorExit_ "Moving $study failed"
		
	done

fi

rmdir "$workDir/studies"


cd $contentDir

echo "Creating DICOMDIR"

DCMMKDIR_FLAGS=""
[ -f "$CD_VAR/dcmmkdir.cfg" ] && . "$CD_VAR/dcmmkdir.cfg"
# [ "$compressionMode" != "AsIs" ] && DCMMKDIR_FLAGS="$DCMMKDIR_FLAGS -Nec -Nrc -Nxc"

if [ "$mediaType" = "CD" -o "$mediaType" = "DVD-RAM" ] ; then
	DCMMKDIR_FLAGS="$DCMMKDIR_FLAGS -Pgp"
	[ "$Compression" != "Uncompressed" ] && DCMMKDIR_FLAGS="$DCMMKDIR_FLAGS -Nec -Nrc -Nxc"
elif [ "$mediaType" = "DVD" ] ; then
	DCMMKDIR_FLAGS="$DCMMKDIR_FLAGS -Pdv"
	[ "$Compression" = "AsIs" ] && DCMMKDIR_FLAGS="$DCMMKDIR_FLAGS -Nec -Nrc -Nxc"
else
# if [ "$mediaType" = "CF" -o "$mediaType" = "MMC" -o "$mediaType" = "SD" -o "$mediaType" = "USB" ] ; then
	DCMMKDIR_FLAGS="$DCMMKDIR_FLAGS -Pfl"
	[ "$Compression" = "AsIs" ] && DCMMKDIR_FLAGS="$DCMMKDIR_FLAGS -Nec -Nrc -Nxc"
fi

[ -z "$mediaID" ] && mediaID=$(sed 's;[^A-Z0-9_];_;g' <<<"EP $mediaType")
echo "$DCMTK_HOME/bin/dcmmkdir +I +W ${DCMMKDIR_FLAGS} --recurse --pattern '[ROB]*' +F "$mediaID" +R README ./"

$DCMTK_HOME/bin/dcmmkdir +I +W ${DCMMKDIR_FLAGS} --recurse --pattern '[ROB]*' +F "$mediaID" +R README ./ 2>&1
[ $? -ne 0 ] && errorExit_ "DICOMDIR could not be created"

dicomdirUID=`$DCMTK_HOME/bin/dcmdump +P 0002,0003 DICOMDIR | awk -vFS="[][]" '{print $2}'`
timeStamp=`date -r DICOMDIR`
# get site ID
siteID=""
[ -f "$CONF/site.conf" ] && . $CONF/site.conf
[ -z "$siteID" ] && siteID="unknownSite"
# get server ID
SELF_SERVER_ID=""
[ -f "$CONF/serverid.cfg" ] && . $CONF/serverid.cfg
[ -z "$SELF_SERVER_ID" ] && SELF_SERVER_ID="unknownServer"

echo -e "MediaUID=\"$dicomdirUID\"\r" > README
echo -e "MediaID=\"$mediaID\"\r" >> README
echo -e "Server=\"$siteID/$SELF_SERVER_ID\"\r" >> README
echo -e "Creator=\"$userID\"\r" >> README
echo -e "Time=\"$timeStamp\"\r" >> README

mediaMD5=`(cat DICOMDIR ; echo -n "$md5Sum" ) | md5sum | awk '{print $1}'`

echo -e "CheckSum=\"$mediaMD5\"\r" >> README

# add viewer

if [ "$addViewer" = "true" ]; then
	cp "$VIEWER_HOME/epcdview.exe" $contentDir/epcdview.exe
	cp "$VIEWER_HOME/epcdview.x64.exe" $contentDir/epcdview.x64.exe

	# add smart start script
	cat <<-ENDOFSTARTVBS > $contentDir/epcdstart.vbs
Set objShell = CreateObject("WScript.Shell")

Set objFSO = CreateObject("Scripting.FileSystemObject")
scriptdir = objFSO.GetParentFolderName(objFSO.GetFile(WScript.ScriptFullName))

bits = GetObject("winmgmts:root\cimv2:Win32_Processor='cpu0'").AddressWidth

if bits = 64 then
  objShell.Exec scriptdir & "\epcdview.x64.exe"
else
  objShell.Exec scriptdir & "\epcdview.exe"
end if
	ENDOFSTARTVBS

	# add autorun.inf
	cat <<-ENDOFAUTORUN > $contentDir/autorun.inf
		[autorun]
		icon=epcdview.exe
		label=DICOM Media created by ${PRODUCT_NAME}
		shellexecute=epcdstart.vbs
		action=View
	ENDOFAUTORUN

	profileFile=""
	profileXml=""
	if [ -n "$profileUser" ] ; then
		userDir=$( ${REPOSITORYHANDLER_HOME}/bin/locateResource "${REPOSITORY_USERS}" "${profileUser}" )
		[ -n "${userDir}" -a -f "${userDir}/profile.ini" ] && profileFile="${userDir}/profile.ini"
		[ -n "${userDir}" -a -f "${userDir}/tempProfile.xml" ]  && profileXml="${userDir}/tempProfile.xml"
		fi
	if [ -z "${profileFile}" ] ; then
		userDir=$( ${REPOSITORYHANDLER_HOME}/bin/locateResource "${REPOSITORY_USERS}" "${userID}" )
		[ -n "${userDir}" -a -f "${userDir}/profile.ini" ]  && profileFile="${userDir}/profile.ini"
	fi
	if [ -z "${profileXml}" ] ; then
		userDir=$( ${REPOSITORYHANDLER_HOME}/bin/locateResource "${REPOSITORY_USERS}" "${userID}" )
		[ -n "${userDir}" -a -f "${userDir}/tempProfile.xml" ]  && profileXml="${userDir}/tempProfile.xml"
	fi
	[ -f "${profileFile}" ] && cp "${profileFile}" "$contentDir/"
	[ -f "${profileXml}" ] && mv "${profileXml}" "$contentDir/profile.xml"
				
				
	if [ -f "$languageFile" ] ; then
		cp "$languageFile" "$contentDir/Viewer.properties"
	fi
	
	
fi

if [ "$mediaType" = "CD" -o "$mediaType" = "DVD-RAM" -o "$mediaType" = "DVD" ] ; then
	echo "Creating ISO file"
	mkisofs -l -J -r -v -o "$workDir"/cdimage.iso ./ 2>&1
	[ $? -ne 0 ] && errorExit_ "ISO file could not be created"
else
	echo "Creating ZIP file"
	zip -r "$workDir"/media.zip * 2>&1
	[ $? -ne 0 ] && errorExit_ "ZIP file could not be created"
fi
echo DONE > $workDir/cdimage.done


for s in $( awk -vRS="|" '{print}' <<<"$Studies" ); do
	$UTILS_HOME/bin/operlog -t "createcd" -l STUDY -a export -s "$s" -f "USERID:$userID" -f "LOGINFORMATION:LABEL: $mediaID; UID: $dicomdirUID"
done
